# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fesh2', 'fesh2.excluded', 'fesh2.schedules', 'fesh2.util']

package_data = \
{'': ['*']}

install_requires = \
['ConfigArgParse>=1.5.3,<2.0.0',
 'additional-urwid-widgets>=0.4.1,<0.5.0',
 'numpy>=1.23.0,<2.0.0',
 'pexpect>=4.8.0,<5.0.0',
 'psutil>=5.9.1,<6.0.0',
 'pycurl>=7.45.1,<8.0.0',
 'urwid>=2.1.2,<3.0.0']

entry_points = \
{'console_scripts': ['fesh2 = fesh2.__main__:main']}

setup_kwargs = {
    'name': 'fesh2',
    'version': '2.3.2',
    'description': 'Geodetic VLBI schedule management and processing',
    'long_description': '# Fesh2: Geodetic VLBI schedule file management and processing\n\n## What is it?\n\n**Fesh2** is a Python package that provides automated schedule file preparation.\nIt requires an installation of the\n[NASA Field System](https://github.com/nvi-inc/fs).\n\n**Fesh2** runs in a terminal window and regularly checks IVS schedule\nrepositories for new or updated versions of Master files and schedule files for\none or more specified stations. A check for the latest version of the Master\nfile(s) is done first, but skipped if the time since the last check is less than\na specified amount.\nSimilarly, checks on schedule files are only done if the time since the last\ncheck exceeds a specified time. If new or updated schedules are found, they are\noptionally processed with **Drudg**\nto produce `snp`, `prc` and `lst` files. By default, once the files have been\nchecked, **fesh2** will provide a summary and then go into a wait state before\ncarrying out another check. **Fesh2** can also be run once for a single \ncheck or\nstatus report and not go into a wait state. **Fesh2** can also be run \nin a `monit` mode that gives a continually updating status report but \ndoes not process any files. Multiple \ninstances can be run\nsimultaneously. If drudg output (`snp` or `prc\n` files) have been modified by the user and a new schedule becomes available\n, **fesh2** will download the file but not overwrite Drudg output, but it will\nwarn the user.\n\n**Fesh2** can be run as a foreground application or as a service in the\nbackground.\n\n## Compatibility\n\n**Fesh2** code is compatible with Python versions 3.8 and above. ***Note that **fesh2** is not compatible with \nPython version 2.***\n\nNote that if you\'re \nusing an old version of Debian (Jessie or earlier) then you will need to \ninstall python 3.8 or higher by hand. This could be done inside a virtual \nenvironment by a user or on the system as root.\n\n## Installation\n\n\n**Fesh2** is a stand-alone add-on to the \n[Field System](https://github.com/nvi-inc/fs) so installation must be \ncarried out separately.\n\n### PycURL dependency\n\n**Fesh2** depends on the python CURL\nlibrary ([PycURL](http://pycurl.io/docs/latest/index.html))\nwhich should be installed automatically when you install fesh2. However, PycURL\ndepends on the Python development libraries, and it won\'t install if they\'re not\nthere. On a Debian machine, \n\n    apt-get install python3-dev\n\nshould do the trick. You may also need these: \n\n    apt-get install libcurl4-openssl-dev libssl-dev\n\nPlease make sure the above dependencies are satisfied before proceeding.\n\nInstallation should be carried out under the `prog` account. You may wish to \ninstall fesh2 within a python virtual environment, but regardless, the \ninstallation commands are the same:\n\n#### Installing with pip:\nRun the following:\n   ```\npip install fesh2   \n```\nYou will then need to edit the **fesh2** configuration file for your station(s). More information on configuration is provided below.\n\nA template configuration file `fesh2.config` is provided in [the fesh2 GitHub \nrepository](https://github.com/nvi-inc/fesh2/) and a copy of the contents is \nin the Appendix at the end of this document. `fesh2.config` should be \n placed in `/usr2/control` and edited to suit your site.\n\n## Configuration\n**Fesh2** will take its configuration in the following priority order from\nlowest to highest:\n\n* Two config files, in this order:\n    * `/usr2/control/skedf.ctl`\n    * `/usr2/control/fesh2.config`\n* Environment variables\n* Command-line parameters\n\nNote that Fesh2 will only run Drudg if it DoDrudg is set to True and it is\nconfigured appropriately. See the Drudg notes below.\n\nIf you have the configuration files in a different location to `/usr2/control`,\nuse the command-line options `--ConfigFile` and `--SkedConfigFile` to set them\nappropriately.\n\n### Environment Variables\n\nThe following environment variables are recognised by **fesh2**:\n\n* `NETRC_FILE` and `COOKIES_FILE`: the locations of the `.netrc` and\n  `.urs_cookies\n  ` files, used by CURL for the https protocol. Curl sets these to files in the\n  home directory by default. More information is given below on Curl\n  configuration.\n* `LIST_DIR` : the directory where **drudg** puts `.lst` output files. If this\n  is not set, then the `LstDir` parameter in `fesh2.config` is used if set.\n  Otherwise **drudg** defaults to the `$schedules` parameter in `skedf.ctl`\n\nA number of environment variables used by **drudg** are also recognised by \n**fesh2**. See the **drudg** section below.  The \nfollowing \nenvironment variables are _NOT_ recognised by **fesh2**:\n\n* `STATION` is used in the Field System but not recognised by **fesh2** as it is\n  possible to configure **fesh2** to manage schedules for multiple stations .\n  The `Stations` parameter _must_ be set in `fesh2.config`\n\n### The fesh2 configuration file\n\nOn startup, **fesh2** looks for a configuration file called `fesh2.config`. \n*This will need to be set up for your station before running **fesh2** for the\nfirst time.* Use your favourite text editor to modify the file. The comments in\nthe file describe the parameters but here are a few points to note:\n\n#### Station settings\n\nYou will want to edit options in the `[Station]` section to configure which\nstations you want to process, what types of Master and schedule files you want\nto obtain, how far into the future you want to search for schedule files and how\noften you want to look for them.\n\n#### Email\n\nIf enabled, notification can be sent via email when:\n\n* a new schedule has been downloaded\n* a schedule file has been updated on the server, downloaded and needs manual\n  processing\n\n#### Drudg\n\nAfter finding and downloading a new or updated schedule, **Fesh2** can\noptionally run **Drudg** to produce new snp, prc and lst files. The `[Drudg]`\nsection allows you to configure this behaviour. If you don\'t want **fesh2** to\nautomatically process your schedules, this feature con be turned off by\nsetting `DodDrudg` to `False`.\n\nBecause **fesh2** manages schedule files and uses **drudg** to process them, and\nto keep things consistent across the Field System, it will use settings\nin `skedf.ctl`. However, there are some caveats regarding `skedf.ctl` for the\ncase where there is not an overriding definition:\n\n* **Fesh2** requires that `$schedules` is defined in `skedf.ctl`. The default\n  of \'.\' (i.e. the directory the software is started in) is too arbitrary and\n  could cause **fesh2** to lose track of files.\n* If `$snap` or `$proc` are not defined, then they will be set to the same as\n  `$schedules`\n\nDrudg may have been configured such that the user is prompted for a response, in\nwhich case it cannot be automated. If this is the case, then Drudg will not be\nrun on SKD files.\n\nIf you want Fesh2 to automatically process SKD files, then some configuration is required. Fesh2 will get it\'s configuration information as follows, in\nthis priority order (lowest to highest):\n\n1. Fesh2 will first look in `/usr2/control/skedf.ctl`. Unless parameters are\n   changed, Fesh2 will not run Drudg if the following parameters are set as\n   shown:\n    * `tpicd YES`\n    * `vsi_align ASK`\n    * `cont_cal ASK`\n    * `cont_cal_polarity ASK`\n\n2. Fesh2 will then look for any configuration parameters in the `[Drudg]`\n   secoion of `fesh2.config`. These parameters can be different to `skedf.ctl`\n   in case you want Drudg to behave differently when Fesh2 is running it for you\n\n3. There are some `FESH` environment variables which can be set and these will\n   override skedf.ctl or this file if they are. These environment variables are\n   as follows with the lowest supported version of Drudge shown in brackets (text from FS documentation):\n    * `FESH_GEO_TPICD` *(10.0.0-beta3, December 2020)* Possible values are\n      non-negative integers. If set, the value is provided as the answer for the\n      drudg prompt for the tpicd interval for geodesy schedules.\n    * `FESH_GEO_CONT_CAL` *(10.0.0-beta3, December 2020)* Possible values\n      are `on` or `off`. If set, the value is provided as the answer for the\n      drudg prompt for continuous cal use for geodesy schedules.\n    * `FESH_GEO_CONT_CAL_POLARITY` *(10.0.0-beta3, December 2020)* Possible\n      values are `0`, `1`, `2`, `3,` or `none`. If set, the value is provided as\n      the answer for the drudg prompt for the continuous cal polarity for\n      geodesy schedules.\n    * `FESH_GEO_VSI_ALIGN` *(10.0.0-beta3, December 2020)* Possible values\n      are `0`, `1`, or `none`. If set, the value is provided as the answer for\n      the drudg prompt for using vsi_align for geodesy schedules.\n    * `FESH_GEO_USE_SETUP_PROC` *(10.1.0, 2021)* Possible values are `yes`\n      or `no`. If set, the value is provided as the answer for the drudg prompt\n      for the *"use setup_proc"* for geodesy schedules, an option to write\n      a `.snp` file that skips setup on scans when the mode hasn’t changed.\n    * `FESH_GEO_VDIF_SINGLE_THREAD_PER_FILE` *(10.1.0, 2021)* Possible values\n      are `yes` or `no`. If set, the value is provided as the answer for the\n      drudg prompt for the *"VDIF single thread per file"* for geodesy\n      schedules.\n\n4. Lastly, if any Drudg-related command-line parameters are set, they will\n   override everything else\n\n*If any Drudg settings recieved by Fesh2 require manual intervention, Drudg will\nnot be run on the SKD files.*\n\n#### Servers\n\nThe `[Servers]` section allows you to list all the IVS schedule file servers you\nwant to search and the protocols to use for each. Specify the location of the\ntop directory (i.e. the `vlbi` directory). Protocols known to work are https (\nsecure HTTP), ftp (anonymous FTP) and ftps (secure (SSL) anonymous FTP).\n\n#### Curl\n\n**Fesh2** uses [curl](http://pycurl.io/docs/latest/index.html) to access files\non the servers. If https is being used\n(e.g. to access the CDDIS repository), then **fesh2** needs to know the location\nof your `.netrc` and (\noptionally) `.urs_cookies` files. Curl puts these in the user\'s home directory\nby default but they can be placed elsewhere if desired. This can be set on the\ncommand line or via the NETRC_FILE and COOKIES_FILE environment variables (\nsee above\n) or in the **fesh2** config file by setting the NetrcFile and CookiesFile\nparameters.\n\n## Usage\n\n**Fesh2** can be started by just typing\n```\nfesh2\n```\n\nwhich starts it in its default mode. **fesh2** can also be run in monitoring \nmode in a terminal window by typing:\n```\nfesh2 --monit\n```\n*Note: If you run **fesh2** as multiple instances but with different \nconfiguration (e.g. running it in the background with SnapDir set on the \ncommand line, then running it as a monitor with the default SNAP file \ndirectory) then you may see some unexpected behaviour. In the above \nexample, a schedule could be generated by drudg in the non-default SNAP \ndirectory but ```fesh2 --monit``` would think that the schedule hadn\'t \nbeen processed.*\n\nCommand-line options exist to change many of the configuration parameters, allow\nfor a one-off checks\n(no wait mode), forcing of file downloads etc. Some common usages are:\n\n* Only consider the current or next experiment\n```\nfesh2 -n\n```\n* Just run once then exit, don\'t go into a wait loop\n```\nfesh2 --once\n```\n* Just get a schedule for a specified session, then exit. e.g.:\n```\nfesh2 --once -g r1456\n```\n* Force an update to the schedule file when there\'s a new one available to\n  replace the old one. The default behaviour is to give the new file the name\n  ```<code>.skd.new``` and prompt the user to take action. The file will also be\n  drudged if the DoDrudg option is True\n```\nfesh2 --update\n```\n* Obtain schedule files, but don\'t process them:\n```\nfesh2 --DoDrudg False\n```\n* Get a status report. Shows the status of schedule files and when schedule\n  servers were last queried.\n```\nfesh2 --check\n```\n* Run fesh2 with all terminal output suppressed. Useful when running fesh2 as a\n  service.\n```\nfesh2 --quiet\n```\nAll command-line parameters are as follows:\n```\nusage:       fesh2 [-h] [-c CONFIGFILE] [-k SKEDCONFIGFILE] [-g GET] [-m] [-u]\n                   [-n] [-a] [-o] [--update] [--SchedDir SCHEDDIR]\n                   [--ProcDir PROCDIR] [--SnapDir SNAPDIR] [--LstDir LSTDIR]\n                   [--LogDir LOGDIR] [--Stations [STATIONS [STATIONS ...]]]\n                   [--EmailNotifications [EMAILNOTIFICATIONS]]\n                   [--GetMaster [GETMASTER]]\n                   [--GetMasterIntensive [GETMASTERINTENSIVE]]\n                   [--SchedTypes [SCHEDTYPES [SCHEDTYPES ...]]]\n                   [-t MASTERCHECKTIME] [-s SCHEDULECHECKTIME]\n                   [-l LOOKAHEADTIMEDAYS] [-d [DODRUDG]]\n                   [--DrudgBinary DRUDGBINARY] [--TpiPeriod TPIPERIOD]\n                   [--VsiAlign VSIALIGN] [--ContCalAction CONTCALACTION]\n                   [--ContCalPolarity CONTCALPOLARITY] [--SetupProc SETUPPROC]\n                   [--VdifSingleThreadPerFile VDIFSINGLETHREADPERFILE]\n                   [--Servers [SERVERS [SERVERS ...]]] [--NetrcFile NETRCFILE]\n                   [--CookiesFile COOKIESFILE]\n                   [--CurlSecLevel1 [CURLSECLEVEL1]] [-y YEAR] [-e] [--monit]\n                   [-q]\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -c CONFIGFILE, --ConfigFile CONFIGFILE\n                        The fesh2 configuration file to use. e.g.\n                        /usr2/control/fesh2.config\n  -k SKEDCONFIGFILE, --SkedConfigFile SKEDCONFIGFILE\n                        The location of the skedf.cfg configuration file\n  -g GET, --get GET     Just get a schedule for this specified session. Give\n                        the name of the session (e.g. r4951).\n  -m, --master-update   Force a download of the Master Schedule (default =\n                        False), but just on the first check cycle.\n  -u, --sched-update    Force a download of the Schedules (default = False),\n                        but just on the first check cycle.\n  -n, --current, --now  Only process the current or next experiment\n  -a, --all             Find the experiments with all "Stations" in it\n  -o, --once            Just run once then exit, don\'t go into a wait loop\n                        (default = False)\n  --update              Force an update to the schedule file when there\'s a\n                        new one available to replace the old one. The default\n                        behaviour is to give the new file the name\n                        <code>.skd.new and prompt the user to take action. The\n                        file will also be drudged if the DoDrudg option is\n                        True\n  --SchedDir SCHEDDIR   Schedule file directory\n  --ProcDir PROCDIR     Procedure (PRC) file directory\n  --SnapDir SNAPDIR     SNAP file directory\n  --LstDir LSTDIR       LST file directory [env var: LIST_DIR]\n  --LogDir LOGDIR       Log file directory\n  --Stations [STATIONS [STATIONS ...]]\n                        Stations to consider (e.g. "hb ke yg ho", "mg")\n  --EmailNotifications [EMAILNOTIFICATIONS]\n                        Send notifications by email. The fesh2 config file\n                        will be read for details on mail server, recipients\n                        etc\n  --GetMaster [GETMASTER]\n                        Maintain a local copy of the main Multi-Agency\n                        schedule, i.e. mostly 24h sessions (default = True)\n  --GetMasterIntensive [GETMASTERINTENSIVE]\n                        Maintain a local copy of the main Multi-Agency\n                        Intensive schedule (default = True)\n  --SchedTypes [SCHEDTYPES [SCHEDTYPES ...]]\n                        Schedule file formats to be obtained? This is a\n                        prioritised list with the highest priority first. Use\n                        the file name suffix ("vex" and/or "skd") and comma-\n                        separated.\n  -t MASTERCHECKTIME, --MasterCheckTime MASTERCHECKTIME\n                        Only check for a new master file if the last check was\n                        more than this number of hours ago. The default is set\n                        in the configuration file.\n  -s SCHEDULECHECKTIME, --ScheduleCheckTime SCHEDULECHECKTIME\n                        Only check for a new schedule file (SKD or VEX) if the\n                        last check was more than this number of hours ago. The\n                        default is set in the configuration file.\n  -l LOOKAHEADTIMEDAYS, --LookAheadTimeDays LOOKAHEADTIMEDAYS\n                        Only look for schedules less than this number of days\n                        away (default is 7)\n  -d [DODRUDG], --DoDrudg [DODRUDG]\n                        Run Drudg on the downloaded/updated schedules (default\n                        = True)\n  --DrudgBinary DRUDGBINARY\n                        Location of Drudg executable (default =\n                        /usr2/fs/bin/drudg)\n  --TpiPeriod TPIPERIOD\n                        Drudg config: TPI period in centiseconds. 0 = don\'t\n                        use the TPI daemon (default) [env var: FESH_GEO_TPICD]\n  --VsiAlign VSIALIGN   Drudg config: Applicable only for PFB DBBCs, none =\n                        never use dbbc=vsi_align=... (default) 0 = always use\n                        dbbc=vsi_align=0 1 = always use dbbc=vsi_align=1 [env\n                        var: FESH_GEO_VSI_ALIGN]\n  --ContCalAction CONTCALACTION\n                        Drudg config: Continuous cal option. Either \'on\' or\n                        \'off\'. Default is \'off\' [env var: FESH_GEO_CONT_CAL]\n  --ContCalPolarity CONTCALPOLARITY\n                        Drudg config: If continuous cal is in use, what is the\n                        polarity? Options are 0-3 or \'none\'. [env var:\n                        FESH_GEO_CONT_CAL_POLARITY]\n  --SetupProc SETUPPROC\n                        Drudg config: the answer for the drudg prompt for the\n                        use setup_proc for geodesy schedules, an option to\n                        write a `.snp` file that skips setup on scans when the\n                        mode hasn’t changed. [env var:\n                        FESH_GEO_USE_SETUP_PROC]\n  --VdifSingleThreadPerFile VDIFSINGLETHREADPERFILE\n                        VDIF single thread per file: only applies to Mark5C or\n                        Flexbuff recorders. Can be \'yes\' or \'no\' [env var:\n                        FESH_GEO_VDIF_SINGLE_THREAD_PER_FILE]\n  --Servers [SERVERS [SERVERS ...]]\n                        Schedule file server URLs. Each of these will be\n                        checked for the most recent files. Use comma-separated\n                        URLs and specify the top directory (i.e. the \'vlbi\'\n                        directory). Use protocols https (Curl), ftp (anonymous\n                        FTP) or sftp (anonymous secure FTP)\n  --NetrcFile NETRCFILE\n                        The location of the .netrc file, needed by CURL for\n                        the https protocol. (CURL puts this in ~/.netrc by\n                        default) [env var: NETRC_FILE]\n  --CookiesFile COOKIESFILE\n                        The location of the .urs_cookies files used by CURL.\n                        (CURL puts this in ~/.urs_cookies by default) [env\n                        var: COOKIES_FILE]\n  --CurlSecLevel1 [CURLSECLEVEL1]\n                        Workaround for CDDIS https access in some Debian\n                        distributions. See the documentation (default = False)\n  -y YEAR, --year YEAR  The year of the Master Schedule (default is this year)\n  -e, --check           Check the current fesh2 status. Shows the status of\n                        schedule files and when schedule servers were last\n                        queried.\n  --monit               Similar to --check but text output format is intended\n                        for a FS monit interface\n  -q, --quiet           Runs fesh2 with all terminal output suppressed. Useful\n                        when running fesh2 as a service.\n```\n\n## Logging\n\nAs well as writing information to the screen on activity, **fesh2** also keeps a\nlog of activity in the Field System log directory (by default) at\n`/usr2/log/fesh2.log`. The log file location can be configured in `fesh2.config`\n\n## Running fesh2 as a service\n\nFesh2 can be run in the background as a `systemd` service. All output is\nsuppressed and status is available by examining the log file or using the\n`--check` or `-e` flag. Here\'s how to set it up from the superuser account for\nDebian Jessie or later:\n\n1. Type the following command to add a `systemd` service:\n    ```\n   systemctl edit --force --full fesh2.service\n   ```\n   This should open a text editor. Paste in the following:\n    ```\n   [Unit]\n   Description=Fesh2 Service\n   After=network-online.target\n   Wants=network-online.target\n   \n   [Service]\n   ExecStart=/usr/bin/sudo -H -u oper /usr/local/bin/fesh2 --quiet\n   \n   [Install]\n   WantedBy=multi-user.target\n   ```\n   Save and exit. This will configure `systemd` to start fesh2, running as user\n   oper and suppress all output.\n2. Enable the service:\n    ```\n    sudo systemctl enable fesh2.service\n    ```\n3. Check the status of the service:\n    ```\n    sudo systemctl status fesh2.service\n    ```\n4. You can stop, start and query the service:\n    ```\n    sudo systemctl stop fesh2.service          # Stop running the service \n    sudo systemctl start fesh2.service         # Start running the service \n    sudo systemctl restart fesh2.service       # Restart the service \n    ```\n5. To see the current schedule file status (as user oper):\n    ```\n   fesh2 --check\n   ```\n\nIf you would prefer to set this up as a user service, some notes are here:\n* https://www.unixsysadmin.com/systemd-user-services/\n\nFor versions of Debian older thean Jessie (e.g. Wheezy), systemd can be enabled\nand some notes are here:\n* https://scottlinux.com/2014/10/20/how-to-switch-to-systemd-on-debian-wheezy/\n\n# Appendix A: fesh2.config\n\n```\n# Fesh2 configuration file.\n#\n# Note: all non-essential parameters (i.e. those with defaults) are commented out\n#\n# Field System directories\n[FS]\n  # Log directory. This is where the fesh2 log file will be written.\n  # Default is /usr2/log\n  #LogDir = /usr2/log\n\n# Default settings for this station/facility\n[Station]\n  # station name(s) to process by default. Just a two-letter code for a single\n  # station or comma separated two-letter codes inside square brackets for\n  # multiple stations. e.g.:\n  # Stations = [Hb, Ho, Ke, Yg]\n  # Stations = Ke\n  Stations =  Mg\n  #\n  ##################\n  # Which master files should be processed? At least one of these must be selected\n  # Get the main Multi-Agency schedule, i.e. mostly 24h sessions. Default is True\n  #GetMaster = True\n  # Get the Multi-Agency Intensives schedule. Default is True\n  #GetMasterIntensive = True\n  ##################\n  # What schedule file formats should be obtained? This is a prioritised list with the highest one first\n  # Use the file name suffix (vex and/or skd) and comma-separated. Default is skd\n  #SchedTypes = skd\n  #SchedTypes = [skd, vex]\n  # What is the minimum time between checks for a new Master file on the server (hours)?\n  MasterCheckTime = 12\n  # What is the minimum time between checks for a new Schedule (SKD or VEX) file on the server (hours)?\n  ScheduleCheckTime = 1\n  # How far ahead in time should we look for schedule files (days)?\n  LookAheadTimeDays = 14\n\n[Email]\n  # If Fesh2 finds a problem with a schedule or if a schedule needs manual processing,\n  # a notification message can be sent via email. To turn this feature on, set email_notifications to True,\n  # specify a recipient(s)  and configure with the address of the SMTP server.\n  EmailNotifications = True\n  # an array containing one or more email addresses to receive notifications\n  EmailRecipients = [me@gmail.com]\n  EmailSender = my_email_address@thing.com\n  # The SMTP server\n  EmailServer = myserver.thing.com.au\n  # SMTP Port\n  #EmailPort = 587\n  # Server Password. Leave commented out if you don\'t need one\n  #EmailPassword = "VerySecret"\n\n[Drudg]\n  # Drudg-related config\n  # Run drudg on downloaded schedules? If False then files are downloaded but not processed\n  DoDrudg = True\n  # Location of the drudg executable\n  DrudgBinary = /usr2/fs/bin/drudg\n  # Directory for the output SNP file\n  # Default is /usr2/sched\n  SnapDir = /usr2/sched\n  # Directory for the output LST (schedule summary) file\n  # Default is /usr2/sched\n  LstDir = /usr2/sched\n  #-------------------------------------------------------------------------------------------------\n  # If DoDrudg is True, then Fesh2 will run Drudg on the SKD files it downloads. It will drudg all new\n  # SKD files and all updated SKD files provided they have not been changed locally in the meantime.\n  #\n  # However, Drudg may have been configured such that the user is prompted for a response, in which\n  # case it cannot be automated. If this is the case, then Drudg will not be run on SKD files.\n  #\n  # If you want Fesh2 to automatically process SKD files, then some configuration of Drudg is required.\n  # Fesh2 will get it\'s configuration information as follows, in this priority order (lowest to highest):\n  # config file, env var, command line\n  # 1. Fesh2 will first look in /usr2/control/skedf.ctl\n  # 2. It will then look for any configuration parameters in this file (just below these comments)\n  #    These parameters can be different to skedf.ctl in case you want Drudg to behave differently\n  #    when Fesh2 is running Drudg for you\n  # 3. There are some FESH environment variables which can be set and these will override skedf.ctl\n  #    or this file if they are.\n  # 4. Lastly, any Drudg-related command-line parameters are set, they will override everything else\n  #\n  # If any Drudg settings recieved by Fesh2 require manual intervention, Drudg will not be run on\n  # the SKD files.\n  #\n  # If set, these parameters will override the corresponding settings in skedf.ctl but will be overridden\n  # by environment variables and command-line parameters. Uncomment and set them if you wish, otherwise they are\n  # ignored:\n  #\n  #  1. TPI period in centiseconds\n  #     0 = don\'t use the TPI daemon\n  #TpiPeriod = 0\n  #\n  #  2. Continuous cal option. Either "on" or "off". Default is \'off\'\n  #ContCalAction = off\n  #\n  #  3. If continuous cal is in use, what is the polarity? Options are 0-3 or "none". Default is none\n  #ContCalPolarity = none\n  #\n  #  4. For PFB DBBCs only, vsi_align setup\n  #     Applicable only for PFB DBBCs,\n  #     none = never use dbbc=vsi_align=... (default)\n  #     0 = always use dbbc=vsi_align=0\n  #     1 = always use dbbc=vsi_align=1\n  #VsiAlign = none\n  #\n  #  5. If set, the value is provided as the answer for the drudg prompt for the "use setup_proc" for geodesy\n  #     schedules, an option to write a `.snp` file that skips setup on scans when the mode hasn’t changed.\n  #     Possible values are `yes` or `no`.\n  #SetupProc = yes\n  #\n  #  6. If set, the value is provided as the answer for the drudg prompt for the *"VDIF single thread per file"*\n  #     for geodesy schedules.\n  #     Possible values are `yes` or `no`.\n  #VdifSingleThreadPerFile = yes\n  #-------------------------------------------------------------------------------------------------\n\n[Servers]\n  # Schedule file server URLs. Each of these will be checked for the most recent files.\n  # Use comma-separated URLs inside square brackets, and specify the top directory (i.e. the \'vlbi\' directory).\n  # Use protocols https (Curl), ftp (anonymous FTP) or sftp (anonymous secure FTP)\n  Servers = [https://cddis.nasa.gov/archive/vlbi, ftp://ivs.bkg.bund.de/pub/vlbi, ftp://ivsopar.obspm.fr/pub/vlbi]\n  # Alternative CDDIS (anonymous secure FTP):\n  # Servers = ftps://cddis.nasa.gov/archive/vlbi\n\n[Curl]\n  # Curl configuration files.\n  # A .netrc and a .urs_cookies file are needed by Curl for the https protocol. Curl puts these in the user\'s\n  # home directory by default but they can be placed elsewhere if desired. This can be set\n  # on the command line or via the NETRC_FILE and COOKIES_FILE environment variables. The command-line overrides\n  # the environment variable which overrides the config file\n  NetrcFile = /usr2/control/netrc_fesh2\n  # The cookies file is optional and the default is /dev/null\n  CookiesFile = /dev/null\n  # In some recent Debian distributions, a connection to CDDIS will fail with error 35 (see\n  # item 17 in the "CDDIS https Archive Access/File Download FAQ" https://cddis.nasa.gov/About/CDDIS_File_Download_FAQ\n  # .html). You will see a WARNING message if this occurs and downloads from CDDIS will not be possible.\n  # There is a workaround until the CDDIS server is changed to fix the problem: Set the following\n  # parameter to True\n  CurlSecLevel1 = False\n\n```',
    'author': 'Jim Lovell',
    'author_email': 'jejlovell@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
